@@ -9,10 +9,10 @@
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:
- 
+
  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.
- 
+
  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -311,7 +311,7 @@ class JackClient
       return _client ? jack_cpu_load(_client) : 100.0f;
     }
 
-    pthread_t client_thread_id() const
+    jack_native_thread_t client_thread_id() const
     {
       return _client ? jack_client_thread_id(_client) : 0;
     }
@@ -340,6 +340,15 @@ class JackClient
       throw jack_error("jack_process_callback() not implemented!");
     }
 
+    /// JACK sync callback function.
+    virtual int jack_sync_callback(jack_transport_state_t state
+        , jack_position_t *pos)
+    {
+      (void)state;
+      (void)pos;
+      return 1;
+    }
+
     /// JACK shutdown callback.
     /// By default, this is throwing a jack_error exception. If you don't like
     /// this, you can overwrite this function in your derived class.
@@ -393,6 +402,12 @@ class JackClient
       return static_cast<JackClient*>(arg)->jack_process_callback(nframes);
     }
 
+    static int _jack_sync_callback(jack_transport_state_t state
+        , jack_position_t *pos, void* arg)
+    {
+      return static_cast<JackClient*>(arg)->jack_sync_callback(state, pos);
+    }
+
     static void _jack_shutdown_callback(jack_status_t code
         , const char* reason, void* arg)
     {
@@ -465,8 +480,8 @@ JackClient::JackClient(const std::string& name
     , callback_usage_t callback_usage)
   : _client_name(name)
   , _client(nullptr) // will be set after connecting to JACK
-  , _sample_rate(0)  //             -- " -- 
-  , _buffer_size(0)  //             -- " -- 
+  , _sample_rate(0)  //             -- " --
+  , _buffer_size(0)  //             -- " --
 {
   // check if client name is too long
   // jack_client_name_size() returns the size including the terminating \0
@@ -508,6 +523,14 @@ JackClient::JackClient(const std::string& name
       throw jack_error("Could not set process callback function for '"
           + _client_name + "'!");
     }
+
+    // TODO: separate option to disable sync callback?
+
+    if (jack_set_sync_callback(_client, _jack_sync_callback, this))
+    {
+      throw jack_error("Could not set sync callback function for '"
+          + _client_name + "'!");
+    }
   }
 
   jack_on_info_shutdown(_client, _jack_shutdown_callback, this);
@@ -536,6 +559,3 @@ JackClient::JackClient(const std::string& name
 #undef APF_JACKCLIENT_DEBUG_MSG
 
 #endif
-
-// Settings for Vim (http://www.vim.org/), please do not remove:
-// vim:softtabstop=2:shiftwidth=2:expandtab:textwidth=80:cindent
