@@ -9,10 +9,10 @@
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:
- 
+
  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.
- 
+
  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -31,6 +31,7 @@
 #define APF_MIMOPROCESSOR_H
 
 #include <cassert>  // for assert()
+#include <mutex>
 #include <stdexcept>  // for std::logic_error
 
 #include "apf/rtlist.h"
@@ -38,9 +39,10 @@
 #include "apf/misc.h"  // for NonCopyable
 #include "apf/iterator.h" // for *_iterator, make_*_iterator(), cast_proxy_const
 #include "apf/container.h" // for fixed_vector
+#include "apf/threadtools.h" // for ScopedThread
 
-#define APF_MIMOPROCESSOR_TEMPLATES template<typename Derived, typename interface_policy, typename thread_policy, typename query_policy>
-#define APF_MIMOPROCESSOR_BASE MimoProcessor<Derived, interface_policy, thread_policy, query_policy>
+#define APF_MIMOPROCESSOR_TEMPLATES template<typename Derived, typename interface_policy, typename query_policy>
+#define APF_MIMOPROCESSOR_BASE MimoProcessor<Derived, interface_policy, query_policy>
 
 /** Macro to create a @c Process struct and a corresponding member function.
  * @param name Name of the containing class
@@ -82,15 +84,100 @@ struct thread_traits
 class enable_queries
 {
   protected:
-    enable_queries(size_t fifo_size) : _query_fifo(fifo_size) {}
+    enable_queries()
+      // NB: 1 should be sufficient, but it doesn't seem to work:
+      : _query_fifo(2)
+    {}
+
+    template<typename F>
+    void start_query_thread(F& query_function, int usleeptime)
+    {
+      _query_thread = std::make_unique<QueryThread>(_query_fifo, usleeptime);
+
+      // CAUTION: this must be called after interface_policy::activate()!
+      // If not, an infinite recursion happens!
+      _new_query(query_function);
+    }
+
+    // This is supposed to be called from the audio thread
+    void process_query_commands()
+    {
+      _query_fifo.process_commands();
+    }
+
+    void stop_query_thread()
+    {
+      _keep_running = false;
+      // Stop query thread:
+      _query_thread.reset();
+      // Empty query fifo in case there is still some cleanup to do:
+      _query_fifo.deactivate();
+    }
+
+  private:
+    class CleanupFunction
+    {
+      public:
+        CleanupFunction(CommandQueue& fifo) : _fifo(fifo) {};
+        void operator()() { _fifo.cleanup_commands(); }
+
+      private:
+        CommandQueue& _fifo;
+    };
+
+    struct QueryThread : ScopedThread<CleanupFunction>
+    {
+      QueryThread(CommandQueue& fifo, int usleeptime)
+        : ScopedThread<CleanupFunction>(CleanupFunction(fifo), usleeptime)
+      {}
+    };
+
+    template<typename F>
+    class QueryCommand : public CommandQueue::Command
+    {
+      public:
+        QueryCommand(F& query_function, enable_queries& parent)
+          : _query_function(query_function)
+          , _parent(parent)
+        {}
+
+        // This is called in the audio thread
+        virtual void execute()
+        {
+          _query_function.query();
+        }
+
+        // This is called in the query thread
+        virtual void cleanup()
+        {
+          if (_parent._keep_running)
+          {
+            _query_function.update();
+            _parent._new_query(_query_function);  // "recursive" call!
+          }
+        }
+
+      private:
+        F& _query_function;
+        enable_queries& _parent;
+    };
+
+    template<typename F>
+    void _new_query(F& query_function)
+    {
+      _query_fifo.push(new QueryCommand<F>(query_function, *this));
+    }
+
+    std::atomic<bool> _keep_running{true};
+    std::unique_ptr<QueryThread> _query_thread;
     CommandQueue _query_fifo;
 };
 
 class disable_queries
 {
   protected:
-    disable_queries(size_t) {}
-    struct { void process_commands() {} } _query_fifo;
+    void process_query_commands() {}
+    void stop_query_thread() {}
 };
 
 /** Multi-threaded multiple-input-multiple-output (MIMO) processor.
@@ -102,22 +189,19 @@ class disable_queries
  * @tparam Derived Your derived class -> CRTP!
  * @tparam interface_policy Policy class. You can use existing policies (e.g.
  *   jack_policy, pointer_policy<T*>) or write your own policy class.
- * @tparam thread_policy Policy for threads, locks and semaphores.
  *
  * Example: @ref MimoProcessor
  **/
 template<typename Derived
-  , typename interface_policy, typename thread_policy
+  , typename interface_policy
   , typename query_policy = disable_queries>
 class MimoProcessor : public interface_policy
-                    , public thread_policy
                     , public query_policy
                     , public CRTP<Derived>
                     , NonCopyable
 {
   public:
     using typename interface_policy::sample_type;
-    using query_policy::_query_fifo;
 
     class Input;
     class Output;
@@ -174,7 +258,7 @@ class MimoProcessor : public interface_policy
     class ScopedLock : NonCopyable
     {
       public:
-        explicit ScopedLock(typename thread_policy::Lock& obj)
+        explicit ScopedLock(std::mutex& obj)
           : _obj(obj)
         {
           _obj.lock();
@@ -183,58 +267,28 @@ class MimoProcessor : public interface_policy
         ~ScopedLock() { _obj.unlock(); }
 
       private:
-        typename thread_policy::Lock& _obj;
+        std::mutex& _obj;
     };
 
-    class QueryThread
-    {
-      public:
-        QueryThread(CommandQueue& fifo) : _fifo(fifo) {};
-        void operator()() { _fifo.cleanup_commands(); }
-
-      private:
-        CommandQueue& _fifo;
-    };
-
-    template<typename F>
-    class QueryCommand : public CommandQueue::Command
-    {
-      public:
-        QueryCommand(F& query_function, Derived& parent)
-          : _query_function(query_function)
-          , _parent(parent)
-        {}
-
-        virtual void execute()
-        {
-          _query_function.query();
-        }
-
-        virtual void cleanup()
-        {
-          _query_function.update();
-          _parent.new_query(_query_function);  // "recursive" call!
-        }
-
-      private:
-        F& _query_function;
-        Derived& _parent;
-    };
-
-    template<typename F>
-    void new_query(F& query_function)
-    {
-      _query_fifo.push(new QueryCommand<F>(query_function, this->derived()));
-    }
-
     bool activate()
     {
       _fifo.reactivate();  // no return value
       return interface_policy::activate();
     }
 
+    /// This is only available when enable_queries is used
+    template<typename F>
+    bool activate(F& query_function, int usleeptime)
+    {
+      auto result = this->activate();
+      this->start_query_thread(query_function, usleeptime);
+      return result;
+    }
+
     bool deactivate()
     {
+      this->stop_query_thread();
+
       if (!interface_policy::deactivate()) return false;
 
       // All audio threads should be stopped now.
@@ -278,14 +332,10 @@ class MimoProcessor : public interface_policy
     const rtlist_t& get_input_list() const { return _input_list; }
     const rtlist_t& get_output_list() const { return _output_list; }
 
-    const parameter_map params;
+    unsigned threads() const { return _num_threads; }
 
-    template<typename F>
-    static typename thread_policy::template ScopedThread<F>*
-    new_scoped_thread(F f, typename thread_policy::useconds_type usleeptime)
-    {
-      return new typename thread_policy::template ScopedThread<F>(f,usleeptime);
-    }
+    // TODO: make private?
+    const parameter_map params;
 
   protected:
     using MimoProcessorBase = APF_MIMOPROCESSOR_BASE;
@@ -311,58 +361,60 @@ class MimoProcessor : public interface_policy
     CommandQueue _fifo;
 
   private:
-    class WorkerThreadFunction;
-
     class WorkerThread : NonCopyable
     {
-      private:
-        using DetachedThread = typename thread_policy::template DetachedThread<
-          WorkerThreadFunction>;
-
       public:
         WorkerThread(unsigned thread_number, MimoProcessor& parent)
-          : cont_semaphore(0)
-          , wait_semaphore(0)
-          , _thread(WorkerThreadFunction(thread_number, parent, *this))
+          : _thread_number(thread_number)
+          , _parent(parent)
+          , _thread(std::thread(&WorkerThread::_thread_function, this))
         {
           // Set thread priority from interface_policy, if available
           thread_traits<interface_policy
-            , typename DetachedThread::native_handle_type>::set_priority(parent
+            , std::thread::native_handle_type>::set_priority(parent
                 , _thread.native_handle());
         }
 
-        typename thread_policy::Semaphore cont_semaphore;
-        typename thread_policy::Semaphore wait_semaphore;
-
-      private:
-        DetachedThread _thread;  // Thread must be initialized after semaphores
-    };
-
-    class WorkerThreadFunction
-    {
-      public:
-        WorkerThreadFunction(unsigned thread_number, MimoProcessor& parent
-            , WorkerThread& thread)
-          : _thread_number(thread_number)
-          , _parent(parent)
-          , _thread(thread)
-        {}
-
-        void operator()()
+        WorkerThread(WorkerThread&& other)
+          : _parent{other._parent}
         {
-          // wait for main thread
-          _thread.cont_semaphore.wait();
-
-          _parent._process_selected_items_in_current_list(_thread_number);
-
-          // report to main thread
-          _thread.wait_semaphore.post();
+          // WorkerThread must be movable to be stored in a std::vector.
+          // We never actually move it, so this should never be called:
+          throw std::logic_error("This is just a work-around, don't move!");
         }
 
+        ~WorkerThread()
+        {
+          _keep_running.store(false, std::memory_order_release);
+          this->cont_semaphore.post();
+          _thread.join();
+        }
+
+        Semaphore cont_semaphore{0};
+        Semaphore wait_semaphore{0};
+
       private:
+        void _thread_function()
+        {
+          for (;;)
+          {
+            // wait for main audio thread
+            this->cont_semaphore.wait();
+
+            if (!_keep_running.load(std::memory_order_acquire)) { break; }
+
+            _parent._process_selected_items_in_current_list(_thread_number);
+
+            // report to main audio thread
+            this->wait_semaphore.post();
+          }
+        }
+
+        std::atomic<bool> _keep_running{true};
         unsigned _thread_number;
         MimoProcessor& _parent;
-        WorkerThread& _thread;
+
+        std::thread _thread;  // Thread must be initialized after semaphores
     };
 
     class Xput;
@@ -374,7 +426,7 @@ class MimoProcessor : public interface_policy
       _process_list(_input_list);
       typename Derived::Process(this->derived());
       _process_list(_output_list);
-      _query_fifo.process_commands();
+      this->process_query_commands();
     }
 
     void _process_current_list_in_main_thread();
@@ -398,12 +450,11 @@ class MimoProcessor : public interface_policy
 APF_MIMOPROCESSOR_TEMPLATES
 APF_MIMOPROCESSOR_BASE::MimoProcessor(const parameter_map& params_)
   : interface_policy(params_)
-  , query_policy(params_.get("fifo_size", size_t(1024)))
+  , query_policy()
   , params(params_)
   , _fifo(params.get("fifo_size", size_t(1024)))
   , _current_list(nullptr)
-  , _num_threads(params.get("threads"
-        , thread_policy::default_number_of_threads()))
+  , _num_threads(params.get("threads", std::thread::hardware_concurrency()))
   , _input_list(_fifo)
   , _output_list(_fifo)
 {
@@ -603,6 +654,3 @@ class APF_MIMOPROCESSOR_BASE::DefaultOutput : public APF_MIMOPROCESSOR_BASE::Out
 #undef APF_MIMOPROCESSOR_BASE
 
 #endif
-
-// Settings for Vim (http://www.vim.org/), please do not remove:
-// vim:softtabstop=2:shiftwidth=2:expandtab:textwidth=80:cindent
