@@ -30,7 +30,9 @@
 #ifndef SSR_FLEXT_H
 #define SSR_FLEXT_H
 
+#include <filesystem>
 #include <string>
+#include <vector>
 
 #include <flext.h>
 
@@ -48,16 +50,20 @@ class ssr_ ## name : public SsrFlext<renderer> { \
 FLEXT_NEW_DSP_V("ssr_" #name "~", ssr_ ## name)
 
 #include "apf/pointer_policy.h"
-#include "apf/cxx_thread_policy.h"
 
-#include "../src/source.h"
+#include "geometry.h"  // for ssr::quat
+#include <gml/util.hpp>  // for gml::radians()
+
+#if !defined(MAXPDSTRING)
+#define MAXPDSTRING 1000
+#endif
 
 template<typename Renderer>
-class SsrFlext : public flext_dsp
+class SsrFlextBase : public flext_dsp
 {
-  FLEXT_HEADER_S(SsrFlext<Renderer>, flext_dsp, setup)
+  FLEXT_HEADER_S(SsrFlextBase<Renderer>, flext_dsp, setup)
 
-  private:
+  protected:
     // TODO: put these functions somewhere else?
     static bool _get(int& argc, const t_atom*& argv, int& out)
     {
@@ -98,136 +104,12 @@ class SsrFlext : public flext_dsp
       return result;
     }
 
-    apf::parameter_map _get_parameters(int argc, const t_atom* argv)
-    {
-      _in_channels = 0;
-      int threads = 0;
-      std::string first_string = "", second_string = "";
-
-      while (argc > 0)
-      {
-        // Note: IsInt(*argv) doesn't work in Pd
-        if (IsFloat(*argv))
-        {
-          int arg;
-          if (!_get(argc, argv, arg))
-          {
-            throw std::invalid_argument("Numeric arguments must be integers!");
-          }
-          if (!_in_channels)
-          {
-            if (arg < 0)
-            {
-              throw std::invalid_argument("A negative number of sources ... "
-                  "how is this supposed to work?");
-            }
-            _in_channels = arg;
-          }
-          else if (!threads)
-          {
-            if (arg < 0)
-            {
-              throw std::invalid_argument("A negative number of threads ... "
-                  "how is this supposed to work?");
-            }
-            threads = arg;
-          }
-          else
-          {
-            throw std::invalid_argument("Too many numeric arguments!");
-          }
-        }
-        else if (IsSymbol(*argv))
-        {
-          if (first_string == "")
-          {
-            first_string = _get(argc, argv);
-          }
-          else if (second_string == "")
-          {
-            second_string = _get(argc, argv);
-          }
-          else
-          {
-            throw std::invalid_argument("Too many string arguments!");
-          }
-        }
-        else
-        {
-          throw std::invalid_argument("Unsupported argument type!");
-        }
-      }
-
-      if (!_in_channels)
-      {
-        post("%s - first numeric argument should specify number of sources! "
-            "None given, creating one source by default ...", thisName());
-        _in_channels = 1;
-      }
-
-      if (first_string == "")
-      {
-        throw std::invalid_argument(
-            "At least one string must be specified as argument!");
-      }
-
-      apf::parameter_map params;
-
-      params.set("reproduction_setup", first_string);
-      params.set("hrir_file", first_string);
-
-      // TODO: At some point, "prefilter_file" should become part of the
-      // configuration file for the reproduction setup.
-      // As soon as this happens, "second_string" should be removed!
-
-      if (second_string != "")
-      {
-        params.set("prefilter_file", second_string);
-      }
-
-      if (threads)
-      {
-        params.set("threads", threads);
-      }
-
-      params.set("block_size", Blocksize());
-      params.set("sample_rate", Samplerate());
-
-      // TODO: let the user choose those values:
-      params.set("delayline_size", 100000);  // in samples
-      params.set("initial_delay", 1000);  // in samples
-
-      std::string info;
-      for (auto it: params)
-      {
-        info += "\n";
-        info += " * ";
-        info += it.first;
-        info += ": ";
-        info += it.second;
-      }
-      post("%s - trying to start with following options:\n * sources: %d%s"
-          , thisName(), _in_channels, info.c_str());
-
-      return params;
-    }
-
   public:
-    SsrFlext(int argc, const t_atom* argv)
-      : _engine(_get_parameters(argc, argv))
+    SsrFlextBase(apf::parameter_map&& params)
+      : _canvasdir{_init_canvasdir()}
+      , _engine(_update_params(std::move(params)))
     {
       _engine.load_reproduction_setup();
-
-      for (size_t i = 0; i < _in_channels; ++i)
-      {
-        _engine.add_source();
-        AddInSignal();
-      }
-
-      AddOutSignal(_engine.get_output_list().size());
-      _engine.activate();  // start parallel processing (if threads > 1)
-      post("%s - initialization of %s completed, %d outputs available"
-          , thisName(), _engine.name(), CntOut());
     }
 
     static void setup(t_classid c)
@@ -238,12 +120,64 @@ class SsrFlext : public flext_dsp
       post("For more information, visit http://spatialaudio.net/ssr/");
     }
 
+  protected:
+    // NB: This has to be called before _update_params()!
+    std::filesystem::path _init_canvasdir()
+    {
+      char buf[MAXPDSTRING];
+      this->GetCanvasDir(buf, sizeof(buf));
+      return {buf};
+    }
+
+    apf::parameter_map _update_params(apf::parameter_map&& params)
+    {
+      for (const char* key: {"reproduction_setup", "hrir_file", "prefilter_file"})
+      {
+        if (params.has_key(key))
+        {
+          params.set(key, (_canvasdir / params.get<std::string>(key)).string());
+        }
+      }
+
+      params.set("block_size", Blocksize());
+      params.set("sample_rate", Samplerate());
+      std::string info;
+      for (auto it: params)
+      {
+        info += "\n";
+        info += " * ";
+        info += it.first;
+        info += ": ";
+        info += it.second;
+      }
+      post("%s - trying to start with following options:%s"
+          , thisName(), info.c_str());
+      return std::move(params);
+    }
+
+    void _init()
+    {
+      AddOutSignal(_engine.get_output_list().size());
+      _engine.activate();  // start parallel processing (if threads > 1)
+      post("%s - initialization of %s completed, %d outputs available"
+          , thisName(), _engine.name(), CntOut());
+    }
+
   private:
     virtual void CbSignal()
     {
       _engine.audio_callback(Blocksize(), InSig(), OutSig());
     }
 
+    std::string _get_string_id(int numeric_id) const
+    {
+      if (numeric_id < 1 || _source_ids.size() < numeric_id)
+      {
+        return {};
+      }
+      return _source_ids[numeric_id - 1];
+    }
+
     FLEXT_CALLBACK_A(_handle_messages)
     void _handle_messages(const t_symbol* s, int argc, const t_atom* argv)
     {
@@ -265,7 +199,7 @@ class SsrFlext : public flext_dsp
           return;
         }
 
-        auto* source = _engine.get_source(src_id);
+        auto* source = _engine.get_source(_get_string_id(src_id));
 
         if (!source)
         {
@@ -282,13 +216,13 @@ class SsrFlext : public flext_dsp
         }
         else if (cmd2 == "pos")
         {
-          if (argc != 2)
+          if (argc < 2)
           {
-            error("%s - src %d pos must be followed by exactly 2 coordinates!"
+            error("%s - src %d pos must be followed by at least 2 coordinates!"
                 , thisName(), src_id);
             return;
           }
-          float x, y;
+          float x, y, z{};
           if (!_get(argc, argv, x))
           {
             error("%s - x must be a float value!", thisName());
@@ -299,8 +233,20 @@ class SsrFlext : public flext_dsp
             error("%s - y must be a float value!", thisName());
             return;
           }
-          source->position = Position(x, y);
+          if (argc >= 1 && !_get(argc, argv, z))
+          {
+            error("%s - z must be a float value!", thisName());
+            return;
+          }
+          if (argc != 0)
+          {
+            error("%s - src %d pos must be followed by at most 3 coordinates!"
+                , thisName(), src_id);
+            return;
+          }
+          source->position = {x, y, z};
         }
+        // For backwards compatibility, use "rot" for 3D rotations
         else if (cmd2 == "azi")
         {
           if (argc != 1)
@@ -315,24 +261,59 @@ class SsrFlext : public flext_dsp
             error("%s - src azi expects a float value!", thisName());
             return;
           }
-          source->orientation = Orientation(azi);
+          source->rotation = ssr::quat(gml::qrotate(
+              gml::radians(azi), {0.0f, 0.0f, 1.0f}));
         }
-        else if (cmd2 == "gain")
+        else if (cmd2 == "rot")
+        {
+          if (argc < 1)
+          {
+            error("%s - src %d rot must be followed by at least 1 angle!"
+                , thisName(), src_id);
+            return;
+          }
+          float azimuth, elevation{}, roll{};
+          if (!_get(argc, argv, azimuth))
+          {
+            error("%s - src rot expects float values!", thisName());
+            return;
+          }
+          if (argc >= 1 && !_get(argc, argv, elevation))
+          {
+            error("%s - src rot expects float values!", thisName());
+            return;
+          }
+          if (argc >= 1 && !_get(argc, argv, roll))
+          {
+            error("%s - src rot expects float values!", thisName());
+            return;
+          }
+          if (argc != 0)
+          {
+            error("%s - src %d rot must be followed by at most 3 angles!"
+                , thisName(), src_id);
+            return;
+          }
+          source->rotation = ssr::quat(
+              gml::qrotate(gml::radians(azimuth),   {0.0f, 0.0f, 1.0f}) *
+              gml::qrotate(gml::radians(elevation), {1.0f, 0.0f, 0.0f}) *
+              gml::qrotate(gml::radians(roll),      {0.0f, 1.0f, 0.0f}));
+        }
+        else if (cmd2 == "vol")
         {
           if (argc != 1)
           {
-            error("%s - src %d gain must be followed by exactly 1 value!"
+            error("%s - src %d vol must be followed by exactly 1 value!"
                 , thisName(), src_id);
             return;
           }
-          float gain;
-          if (!_get(argc, argv, gain))
+          float volume;
+          if (!_get(argc, argv, volume))
           {
-            error("%s - src %d gain expects a float value!"
-                , thisName(), src_id);
+            error("%s - src %d vol expects a float value!", thisName(), src_id);
             return;
           }
-          source->gain = gain;
+          source->gain = volume;
         }
         else if (cmd2 == "mute")
         {
@@ -350,6 +331,22 @@ class SsrFlext : public flext_dsp
           }
           source->mute = mute;
         }
+        else if (cmd2 == "active")
+        {
+          if (argc != 1)
+          {
+            error("%s - src %d active must be followed by exactly 1 argument!"
+                , thisName(), src_id);
+            return;
+          }
+          bool active;
+          if (!_get(argc, argv, active))
+          {
+            error("%s - src active expects a boolean value!", thisName());
+            return;
+          }
+          source->active = active;
+        }
         else if (cmd2 == "model")
         {
           if (argc != 1)
@@ -364,14 +361,7 @@ class SsrFlext : public flext_dsp
             error("%s - src model expects a string value!", thisName());
             return;
           }
-          Source::model_t model = Source::unknown;
-          if (!apf::str::S2A(model_str, model))
-          {
-            error("%s - couldn't convert model string: %s"
-                , thisName(), model_str.c_str());
-            return;
-          }
-          source->model = model;
+          source->model = model_str;
         }
         else
         {
@@ -406,13 +396,13 @@ class SsrFlext : public flext_dsp
         }
         else if (cmd2 == "pos")
         {
-          if (argc != 2)
+          if (argc < 2)
           {
-            error("%s - ref%s pos must be followed by exactly 2 coordinates!"
+            error("%s - ref%s pos must be followed by at least 2 coordinates!"
                 , thisName(), offset_str);
             return;
           }
-          float x, y;
+          float x, y, z{};
           if (!_get(argc, argv, x))
           {
             error("%s - x must be a float value!", thisName());
@@ -423,15 +413,27 @@ class SsrFlext : public flext_dsp
             error("%s - y must be a float value!", thisName());
             return;
           }
+          if (argc >= 1 && !_get(argc, argv, z))
+          {
+            error("%s - z must be a float value!", thisName());
+            return;
+          }
+          if (argc != 0)
+          {
+            error("%s - ref%s pos must be followed by at most 3 coordinates!"
+                , thisName(), offset_str);
+            return;
+          }
           if (offset)
           {
-            _engine.state.reference_offset_position = Position(x, y);
+            _engine.state.reference_position_offset = {x, y, z};
           }
           else
           {
-            _engine.state.reference_position = Position(x, y);
+            _engine.state.reference_position = {x, y, z};
           }
         }
+        // For backwards compatibility, use "rot" for 3D rotations
         else if (cmd2 == "azi")
         {
           if (argc != 1)
@@ -447,13 +449,61 @@ class SsrFlext : public flext_dsp
                 , thisName(), offset_str);
             return;
           }
+          auto rotation = ssr::quat(
+              gml::qrotate(gml::radians(azi), {0.0f, 0.0f, 1.0f}));
           if (offset)
           {
-            _engine.state.reference_offset_orientation = Orientation(azi);
+            _engine.state.reference_rotation_offset = rotation;
           }
           else
           {
-            _engine.state.reference_orientation = Orientation(azi);
+            _engine.state.reference_rotation = rotation;
+          }
+        }
+        else if (cmd2 == "rot")
+        {
+          if (argc < 1)
+          {
+            error("%s - ref%s rot must be followed by at least 1 angle!"
+                , thisName(), offset_str);
+            return;
+          }
+          float azimuth, elevation{}, roll{};
+          if (!_get(argc, argv, azimuth))
+          {
+            error("%s - ref%s rot expects float values!"
+                , thisName(), offset_str);
+            return;
+          }
+          if (argc >= 1 && !_get(argc, argv, elevation))
+          {
+            error("%s - ref%s rot expects float values!"
+                , thisName(), offset_str);
+            return;
+          }
+          if (argc >= 1 && !_get(argc, argv, roll))
+          {
+            error("%s - ref%s rot expects float values!"
+                , thisName(), offset_str);
+            return;
+          }
+          if (argc != 0)
+          {
+            error("%s - ref%s rot must be followed by at most 3 angles!"
+                , thisName(), offset_str);
+            return;
+          }
+          auto rotation = ssr::quat(
+              gml::qrotate(gml::radians(azimuth),   {0.0f, 0.0f, 1.0f}) *
+              gml::qrotate(gml::radians(elevation), {1.0f, 0.0f, 0.0f}) *
+              gml::qrotate(gml::radians(roll),      {0.0f, 1.0f, 0.0f}));
+          if (offset)
+          {
+            _engine.state.reference_rotation_offset = rotation;
+          }
+          else
+          {
+            _engine.state.reference_rotation = rotation;
           }
         }
         else
@@ -467,8 +517,7 @@ class SsrFlext : public flext_dsp
       {
         if (argc != 1)
         {
-          error("%s - vol must be followed by exactly 1 value (in dB)!"
-              , thisName());
+          error("%s - vol must be followed by exactly 1 value!", thisName());
           return;
         }
         float master_volume;
@@ -504,11 +553,131 @@ class SsrFlext : public flext_dsp
       }
     }
 
-    int _in_channels;
+  protected:
+    std::filesystem::path _canvasdir;
     Renderer _engine;
+    std::vector<std::string> _source_ids;
+};
+
+template<typename Renderer>
+class SsrFlext : public SsrFlextBase<Renderer>
+{
+  public:
+    SsrFlext(int argc, const t_atom* argv)
+      : SsrFlextBase<Renderer>(_get_parameters(argc, argv))
+    {
+      post("%s - trying to create %d sources", this->thisName(), _in_channels);
+      for (size_t i = 0; i < _in_channels; ++i)
+      {
+        auto id = this->_engine.add_source("");
+        this->_engine.get_source(id)->active = true;
+        this->_source_ids.push_back(id);
+        this->AddInSignal();
+      }
+      this->_init();
+    }
+
+  private:
+    apf::parameter_map _get_parameters(int argc, const t_atom* argv)
+    {
+      _in_channels = 0;
+      int threads = 0;
+      std::string first_string = "", second_string = "";
+
+      while (argc > 0)
+      {
+        // Note: IsInt(*argv) doesn't work in Pd
+        if (this->IsFloat(*argv))
+        {
+          int arg;
+          if (!this->_get(argc, argv, arg))
+          {
+            throw std::invalid_argument("Numeric arguments must be integers!");
+          }
+          if (!_in_channels)
+          {
+            if (arg < 0)
+            {
+              throw std::invalid_argument("A negative number of sources ... "
+                  "how is this supposed to work?");
+            }
+            _in_channels = arg;
+          }
+          else if (!threads)
+          {
+            if (arg < 0)
+            {
+              throw std::invalid_argument("A negative number of threads ... "
+                  "how is this supposed to work?");
+            }
+            threads = arg;
+          }
+          else
+          {
+            throw std::invalid_argument("Too many numeric arguments!");
+          }
+        }
+        else if (this->IsSymbol(*argv))
+        {
+          if (first_string == "")
+          {
+            first_string = this->_get(argc, argv);
+          }
+          else if (second_string == "")
+          {
+            second_string = this->_get(argc, argv);
+          }
+          else
+          {
+            throw std::invalid_argument("Too many string arguments!");
+          }
+        }
+        else
+        {
+          throw std::invalid_argument("Unsupported argument type!");
+        }
+      }
+
+      if (!_in_channels)
+      {
+        post("%s - first numeric argument should specify number of sources! "
+            "None given, creating one source by default ...", this->thisName());
+        _in_channels = 1;
+      }
+
+      if (first_string == "")
+      {
+        throw std::invalid_argument(
+            "At least one string must be specified as argument!");
+      }
+
+      apf::parameter_map params;
+
+      params.set("reproduction_setup", first_string);
+      params.set("hrir_file", first_string);
+
+      // TODO: At some point, "prefilter_file" should become part of the
+      // configuration file for the reproduction setup.
+      // As soon as this happens, "second_string" should be removed!
+
+      if (second_string != "")
+      {
+        params.set("prefilter_file", second_string);
+      }
+
+      if (threads)
+      {
+        params.set("threads", threads);
+      }
+
+      // TODO: let the user choose those values:
+      params.set("delayline_size", 100000);  // in samples
+      params.set("initial_delay", 1000);  // in samples
+
+      return params;
+    }
+
+    int _in_channels;
 };
 
 #endif
-
-// Settings for Vim (http://www.vim.org/), please do not remove:
-// vim:softtabstop=2:shiftwidth=2:expandtab:textwidth=80:cindent
