@@ -38,14 +38,15 @@
 #include <stdio.h>
 #include <thread>   // std::this_thread::sleep_for
 #include <chrono>   // std::chrono::seconds
+#include <unistd.h> // isatty
 
 #include "configuration.h"
-#include "posixpathtools.h"
+#include "pathtools.h"
 #include "apf/stringtools.h"
-#include "ssr_global.h" // for ssr::verbose, WARNING(), ...
+#include "ssr_global.h" // for ssr::verbose, SSR_WARNING(), ...
 #include "xmlparser.h"  // TODO: move this somewhere else
 
-using posixpathtools::make_path_relative_to_current_dir;
+using pathtools::make_path_relative_to_current_dir;
 using apf::str::S2A;
 
 // only needed in this file
@@ -81,6 +82,9 @@ namespace // anonymous
 #ifdef ENABLE_IP_INTERFACE
         "IP interface|"
 #endif
+#ifdef ENABLE_WEBSOCKET_INTERFACE
+        "WebSocket interface|"
+#endif
 #ifdef ENABLE_INTERSENSE
         "InterSense "
 #ifdef HAVE_INTERSENSE_404
@@ -123,9 +127,11 @@ ssr::conf_struct ssr::configuration(int& argc, char* argv[])
 
   conf_struct conf;
 
+  conf.exec_name = argv[0];
+
 #ifndef NDEBUG
   // Because of this warning, "make check" fails for debug builds (on purpose).
-  WARNING(argv[0] << " was compiled for debugging!");
+  SSR_WARNING(conf.exec_name << " was compiled for debugging!");
 #endif
 
   // hard coded default values:
@@ -134,12 +140,14 @@ ssr::conf_struct ssr::configuration(int& argc, char* argv[])
 #else
   conf.gui = false;
 #endif
-#ifdef ENABLE_IP_INTERFACE
-  conf.ip_server = true;
-#else
   conf.ip_server = false;
-#endif
   conf.server_port = 4711;
+  conf.websocket_server = false;
+  conf.websocket_port = 9422;
+  conf.fudi_server = false;
+  conf.fudi_port = 1174;
+
+  conf.follow = false;
 
   conf.freewheeling = false;
   conf.scene_file_name = "";
@@ -151,6 +159,7 @@ ssr::conf_struct ssr::configuration(int& argc, char* argv[])
   conf.input_port_prefix = "system:capture_";
   conf.output_port_prefix = "system:playback_";
   conf.path_to_gui_images = SSR_DATA_DIR"/images";
+  conf.websocket_resource_directory = SSR_DATA_DIR"/websocket_resources";
   conf.path_to_scene_menu = "./scene_menu.conf";
   conf.end_of_message_character = 0; // default: binary zero
 
@@ -193,7 +202,7 @@ ssr::conf_struct ssr::configuration(int& argc, char* argv[])
   load_config_file(filename.c_str(),conf);
 
   const std::string usage_string =
-"Usage: " + std::string(argv[0]) + " [OPTIONS] <scene-file>\n";
+"Usage: " + std::string(conf.exec_name) + " [OPTIONS] <scene-file>\n";
 
   const std::string help_string =
 "\nThe SoundScape Renderer (SSR) is a tool for real-time "
@@ -235,17 +244,18 @@ ssr::conf_struct ssr::configuration(int& argc, char* argv[])
 "      --master-volume-correction=VALUE\n"
 "                      Correction of the master volume in dB "
                                                          "(default: 0 dB)\n"
-"      --auto-rotation Auto-rotate sound sources' orientation toward "
-                                                               "the reference\n"
+"      --auto-rotation Auto-rotate sound sources' orientation toward the\n"
+"                      reference\n"
 "      --no-auto-rotation\n"
-"                      Don't auto-rotate sound sources' orientation toward "
-                                                               "the reference\n"
+"                      Don't auto-rotate sound sources' orientation "
+                                                                  "toward the\n"
+"                      reference\n"
 
 #ifdef ENABLE_IP_INTERFACE
 "  -i, --ip-server[=PORT]\n"
-"                      Start IP server (default on),\n"
+"                      Start IP server (default off),\n"
 "                      a port number can be specified (default 4711)\n"
-"  -I, --no-ip-server  Don't start IP server\n"
+"  -I, --no-ip-server  Don't start IP server (default)\n"
 "      --end-of-message-character=VALUE\n"
 "                      ASCII code for character to end messages with\n"
 "                      (default 0 = binary zero)\n"
@@ -253,7 +263,33 @@ ssr::conf_struct ssr::configuration(int& argc, char* argv[])
 "  -i, --ip-server     Start IP server (not enabled at compile time!)\n"
 "  -I, --no-ip-server  Don't start IP server (default)\n"
 #endif
-#ifdef ENABLE_GUI      
+#ifdef ENABLE_WEBSOCKET_INTERFACE
+"      --websocket-server[=PORT]\n"
+"                      Start WebSocket server (default off),\n"
+"                      a port number can be specified (default 9422)\n"
+"      --no-websocket-server\n"
+"                      Don't start WebSocket server (default)\n"
+#else
+"      --websocket-server\n"
+"                      Start WebSocket server (not enabled at compile time!)\n"
+"      --no-websocket-server\n"
+"                      Don't start WebSocket server (default)\n"
+#endif
+#ifdef ENABLE_FUDI_INTERFACE
+"      --fudi-server[=PORT]\n"
+"                      Start FUDI server (default off),\n"
+"                      a port number can be specified (default 1174)\n"
+"      --no-fudi-server\n"
+"                      Don't start FUDI server (default)\n"
+#else
+"      --fudi-server\n"
+"                      Start FUDI server (not enabled at compile time!)\n"
+"      --no-fudi-server\n"
+"                      Don't start FUDI server (default)\n"
+#endif
+"      --follow        Wait for another SSR instance to connect\n"
+"      --no-follow     Don't follow another SSR instance (default)\n"
+#ifdef ENABLE_GUI
 "  -g, --gui           Start GUI (default)\n"
 "  -G, --no-gui        Don't start GUI\n"
 #else
@@ -326,6 +362,12 @@ ssr::conf_struct ssr::configuration(int& argc, char* argv[])
     {"ip-server",    optional_argument, nullptr, 'i'},
     {"no-ip-server", no_argument,       nullptr, 'I'},
     {"end-of-message-character", required_argument, nullptr, 0},
+    {"websocket-server", optional_argument, nullptr, 0},
+    {"no-websocket-server", no_argument, nullptr, 0},
+    {"fudi-server",  optional_argument, nullptr,  0 },
+    {"no-fudi-server", no_argument,     nullptr,  0 },
+    {"follow",       no_argument,       nullptr,  0 },
+    {"no-follow",    no_argument,       nullptr,  0 },
     {"gui",          no_argument,       nullptr, 'g'},
     {"no-gui",       no_argument,       nullptr, 'G'},
     {"tracker",      required_argument, nullptr, 't'},
@@ -409,9 +451,51 @@ ssr::conf_struct ssr::configuration(int& argc, char* argv[])
         {
           if (!S2A(optarg, conf.end_of_message_character))
           {
-            ERROR("Invalid end-of-message character specified!");
+            SSR_ERROR("Invalid end-of-message character specified!");
           }
         }
+        else if (strcmp("websocket-server", longopts[longindex].name) == 0)
+        {
+          conf.websocket_server = true;
+#ifdef ENABLE_WEBSOCKET_INTERFACE
+          if (optarg)
+          {
+            if (!S2A(optarg, conf.websocket_port))
+            {
+              SSR_ERROR("Invalid WebSocket port specified!");
+            }
+          }
+#endif
+        }
+        else if (strcmp("no-websocket-server", longopts[longindex].name) == 0)
+        {
+          conf.websocket_server = false;
+        }
+        else if (strcmp("fudi-server", longopts[longindex].name) == 0)
+        {
+          conf.fudi_server = true;
+#ifdef ENABLE_FUDI_INTERFACE
+          if (optarg)
+          {
+            if (!S2A(optarg, conf.fudi_port))
+            {
+              SSR_ERROR("Invalid FUDI port specified!");
+            }
+          }
+#endif
+        }
+        else if (strcmp("no-fudi-server", longopts[longindex].name) == 0)
+        {
+          conf.fudi_server = false;
+        }
+        else if (strcmp("follow", longopts[longindex].name) == 0)
+        {
+          conf.follow = true;
+        }
+        else if (strcmp("no-follow", longopts[longindex].name) == 0)
+        {
+          conf.follow = false;
+        }
         else if (strcmp("tracker-port", longopts[longindex].name) == 0)
         {
           conf.tracker_ports = optarg;
@@ -428,9 +512,9 @@ ssr::conf_struct ssr::configuration(int& argc, char* argv[])
         }
         else
         {
-          ERROR("For now, only one non-option parameter "
+          SSR_ERROR("For now, only one non-option parameter "
               "(= scene file) is allowed!");
-          WARNING("Ignoring '" << optarg << "'.");
+          SSR_WARNING("Ignoring '" << optarg << "'.");
         }
         break;
 
@@ -468,7 +552,7 @@ ssr::conf_struct ssr::configuration(int& argc, char* argv[])
         {
           if (!S2A(optarg, conf.server_port))
           {
-            ERROR("Invalid server port specified!");
+            SSR_ERROR("Invalid server port specified!");
           }
         }
 #endif
@@ -513,7 +597,7 @@ ssr::conf_struct ssr::configuration(int& argc, char* argv[])
         // here we deal with unknown/invalid options
         // getopt() already prints an error message for unknown options
         std::cout << usage_string;
-        std::cout << "Type '" << argv[0] << " --help' "
+        std::cout << "Type '" << conf.exec_name << " --help' "
           "for more information.\n\n";
         exit(EXIT_FAILURE);
         break;
@@ -539,10 +623,10 @@ ssr::conf_struct ssr::configuration(int& argc, char* argv[])
     conf.renderer_params.set("system_output_prefix", conf.output_port_prefix);
   }
 
-  VERBOSE2("Requested renderer settings:");
+  SSR_VERBOSE2("Requested renderer settings:");
   for (const auto& entry: conf.renderer_params)
   {
-    VERBOSE2(entry.first << " = " << entry.second);
+    SSR_VERBOSE2(entry.first << " = " << entry.second);
   }
 
   return conf;
@@ -675,7 +759,7 @@ int ssr::load_config_file(const char *filename, conf_struct& conf){
         break;
     }//switch
 
-    VERBOSE2(key << " = " << value);
+    SSR_VERBOSE2(key << " = " << value);
 
     if (!strcmp(key, "NAME"))
     {
@@ -727,7 +811,7 @@ int ssr::load_config_file(const char *filename, conf_struct& conf){
     }
     else if (!strcmp(key, "RENDERER_TYPE"))
     {
-      WARNING("\"RENDERER_TYPE\" is deprecated, don't use it anymore!");
+      SSR_WARNING("\"RENDERER_TYPE\" is deprecated, don't use it anymore!");
     }
     else if (!strcmp(key, "WFS_PREFILTER"))
     {
@@ -764,7 +848,7 @@ int ssr::load_config_file(const char *filename, conf_struct& conf){
       else if (!strcasecmp(value,"true")) conf.renderer_params.set("in_phase", true);
       else if (!strcasecmp(value,"FALSE")) conf.renderer_params.set("in_phase", false);
       else if (!strcasecmp(value,"false")) conf.renderer_params.set("in_phase", false);
-      else ERROR("I don't understand the option '" << value
+      else SSR_ERROR("I don't understand the option '" << value
           << "' for in-phase rendering.");
     }
     else if (!strcmp(key, "INPUT_PREFIX"))
@@ -780,6 +864,11 @@ int ssr::load_config_file(const char *filename, conf_struct& conf){
       conf.path_to_gui_images
         = make_path_relative_to_current_dir(value, filename);
     }
+    else if (!strcmp(key, "WEBSOCKET_RESOURCE_DIRECTORY"))
+    {
+      conf.websocket_resource_directory
+        = make_path_relative_to_current_dir(value, filename);
+    }
     else if (!strcmp(key, "FREEWHEEL"))
     {
       if (!strcasecmp(value, "yes")) conf.freewheeling = true;
@@ -805,13 +894,50 @@ int ssr::load_config_file(const char *filename, conf_struct& conf){
     }
     else if (!strcmp(key, "END_OF_MESSAGE_CHARACTER"))
     {
-      #ifdef ENABLE_IP_INTERFACE     
+      #ifdef ENABLE_IP_INTERFACE
       if (!S2A(value, conf.end_of_message_character))
       {
-        ERROR("Invalid end-of-message character specified!");
+        SSR_ERROR("Invalid end-of-message character specified!");
       }
       #endif
     }
+    else if (!strcmp(key, "WEBSOCKET_INTERFACE"))
+    {
+      #ifdef ENABLE_WEBSOCKET_INTERFACE
+      if (!strcasecmp(value, "on")) conf.websocket_server= true;
+      else conf.websocket_server= false;
+      #endif
+    }
+    else if (!strcmp(key, "WEBSOCKET_PORT"))
+    {
+      #ifdef ENABLE_WEBSOCKET_INTERFACE
+      conf.websocket_port = atoi(value);
+      #endif
+    }
+    else if (!strcmp(key, "FUDI_INTERFACE"))
+    {
+      #ifdef ENABLE_FUDI_INTERFACE
+      if (!strcasecmp(value, "on")) conf.fudi_server= true;
+      else conf.fudi_server= false;
+      #endif
+    }
+    else if (!strcmp(key, "FUDI_PORT"))
+    {
+      #ifdef ENABLE_FUDI_INTERFACE
+      conf.fudi_port = atoi(value);
+      #endif
+    }
+    else if (!strcmp(key, "FOLLOW"))
+    {
+      if (!strcasecmp(value, "yes"))
+      {
+        conf.follow = true;
+      }
+      else
+      {
+        conf.follow = false;
+      }
+    }
     else if (!strcmp(key, "VERBOSE"))
     {
       ssr::verbose = atoi(value);
@@ -837,6 +963,3 @@ int ssr::load_config_file(const char *filename, conf_struct& conf){
   }//while
   return EXIT_SUCCESS;
 }
-
-// Settings for Vim (http://www.vim.org/), please do not remove:
-// vim:softtabstop=2:shiftwidth=2:expandtab:textwidth=80:cindent
