@@ -27,6 +27,7 @@
 /// @file
 /// TODO: add description
 #include <string>
+#include <filesystem>
 #include <fstream>
 #include <cmath>
 
@@ -40,6 +41,7 @@
 #include <QtWidgets/QMenu>
 #include <QtWidgets/QMessageBox>
 #include <QtWidgets/QVBoxLayout>
+#include <QMimeData>
 
 #include "quserinterface.h"
 #include "apf/math.h"
@@ -48,7 +50,7 @@ using apf::math::dB2linear;
 using apf::math::linear2dB;
 
 #include "apf/stringtools.h"
-#include "posixpathtools.h"
+#include "legacy_scene.h"  // for LegacyScene
 
 #define FILEMENUWIDTH 128
 #define BETWEENBUTTONSPACE 6
@@ -73,6 +75,8 @@ using apf::math::linear2dB;
 // given by qvolumeslider
 #define MINVOLUME -50.0f
 
+namespace fs = std::filesystem;
+
 /** ctor.
  * @param controller this can be either the Controller or, if the IP interface
  * is used, an IPClient.
@@ -81,7 +85,8 @@ using apf::math::linear2dB;
  * @param parent parent Qt widget. If left as \a NULL (default) then the window
  * is startet as an independent main window.
  **/
-ssr::QUserInterface::QUserInterface(Publisher& controller, const Scene& scene
+ssr::QUserInterface::QUserInterface(api::Publisher& controller
+        , const LegacyScene& scene
         , const std::string& path_to_gui_images
         , const std::string& path_to_scene_menu
         , unsigned int update_frequency
@@ -92,7 +97,7 @@ ssr::QUserInterface::QUserInterface(Publisher& controller, const Scene& scene
     _controlsParent(this)
 {
   // set window title
-  std::string type = _controller.get_renderer_name();
+  std::string type = _scene.get_renderer_name();
   if      (type == "wfs")        setWindowTitle("SSR - WFS");
   else if (type == "binaural")   setWindowTitle("SSR - Binaural");
   else if (type == "brs")        setWindowTitle("SSR - BRS");
@@ -113,10 +118,13 @@ ssr::QUserInterface::QUserInterface(Publisher& controller, const Scene& scene
 
   // set default size
   setGeometry(200, 100, 900, 800);
-  
+
   // TODO: use screen size for initial window positions
   //QRect screenSize = QApplication::desktop()->screenGeometry();
   setGeometry(200, 70, 900, 700);
+
+  setAcceptDrops(true);
+
   _controlsParent = new QLabel(this, Qt::Window | Qt::CustomizeWindowHint | Qt::WindowTitleHint);
   _controlsParent->setFixedSize(900, 75);
   _controlsParent->move(200, 780);
@@ -130,7 +138,7 @@ ssr::QUserInterface::QUserInterface(Publisher& controller, const Scene& scene
   connect(_source_properties, SIGNAL(signal_set_source_position_fixed(bool)), this, SLOT(_set_source_position_fixed(bool)));
   connect(_source_properties, SIGNAL(signal_set_source_model(int)), this, SLOT(_set_source_model(int)));
   _source_properties->hide();
-  
+
   // set window icon
   QString path_to_image( _path_to_gui_images.c_str() ) ;
   setWindowIcon(QIcon(QPixmap(path_to_image.append("/ssr_logo_large.png"))));
@@ -234,7 +242,7 @@ ssr::QUserInterface::QUserInterface(Publisher& controller, const Scene& scene
   _deselect_all_sources(); // no source selected
 
   // scene menu is not shown if floating control panel is used
-  VERBOSE("Floating control panel is used, scene menu will not be shown.");
+  SSR_VERBOSE("Floating control panel is used, scene menu will not be shown.");
   (void)path_to_scene_menu;
 
   // update screen with update_frequency
@@ -255,7 +263,7 @@ ssr::QUserInterface::~QUserInterface()
   // clear memory if scene button list has been in use
   if (!_scene_button_list.empty())
   {
-    for (scene_button_list_t::iterator i = _scene_button_list.begin(); 
+    for (scene_button_list_t::iterator i = _scene_button_list.begin();
 	 i != _scene_button_list.end(); i++)
     {
       delete *i;
@@ -268,7 +276,7 @@ ssr::QUserInterface::~QUserInterface()
 /// Skips back to the beginning of the scene
 void ssr::QUserInterface::_skip_back()
 {
-  _controller.transport_locate(0);
+  _transport_locate(0);
 }
 
 /** Skips the scene to a specified instant of time
@@ -278,9 +286,9 @@ void ssr::QUserInterface::_transport_locate(float time)
 {
   if (time >= 0.0f)
   {
-    _controller.transport_locate(time);
+    _controller.take_control()->transport_locate_seconds(time);
   }
-  else 
+  else
   {
     _skip_back();
   }
@@ -294,14 +302,16 @@ void ssr::QUserInterface::_solo_selected_sources()
 {
   _soloed_sources.clear();
 
-  for (selected_sources_map_t::iterator i = _selected_sources_map.begin(); 
+  auto control = _controller.take_control();
+
+  for (selected_sources_map_t::iterator i = _selected_sources_map.begin();
        i != _selected_sources_map.end(); i++)
   {
     _soloed_sources.insert(i->second);
 
     // make sure it's not muted
-    _controller.set_source_mute(i->second, false);
-   
+    control->source_mute(_scene.string_id(i->second), false);
+
   } // for
 
   // get sources
@@ -309,14 +319,14 @@ void ssr::QUserInterface::_solo_selected_sources()
   _scene.get_sources(source_buffer_list);
 
   // mute other sources
-  for (source_buffer_list_t::const_iterator i = source_buffer_list.begin(); 
+  for (source_buffer_list_t::const_iterator i = source_buffer_list.begin();
        i != source_buffer_list.end(); i++)
   {
     // if source is not soloed
     if (_soloed_sources.find(i->id) == _soloed_sources.end())
     {
       // then mute it
-      _controller.set_source_mute(i->id, true);
+      control->source_mute(_scene.string_id(i->id), true);
     }
   }
 
@@ -326,7 +336,7 @@ void ssr::QUserInterface::_solo_selected_sources()
 /// this function is not so useful
 void ssr::QUserInterface::_unsolo_selected_sources()
 {
-  for (selected_sources_map_t::iterator i = _selected_sources_map.begin(); 
+  for (selected_sources_map_t::iterator i = _selected_sources_map.begin();
        i != _selected_sources_map.end(); i++)
   {
     _soloed_sources.erase(i->second);
@@ -340,11 +350,12 @@ void ssr::QUserInterface::_unsolo_selected_sources()
   // if other sources are soloed
   else
   {
-    for (selected_sources_map_t::iterator i = _selected_sources_map.begin(); 
+    auto control = _controller.take_control();
+    for (selected_sources_map_t::iterator i = _selected_sources_map.begin();
 	 i != _selected_sources_map.end(); i++)
     {
       // then mute the unsoloed sources
-      _controller.set_source_mute(i->second, true); 
+      control->source_mute(_scene.string_id(i->second), true);
     } // for
   }
 
@@ -358,36 +369,30 @@ void ssr::QUserInterface::_unsolo_all_sources()
   source_buffer_list_t source_buffer_list;
   _scene.get_sources(source_buffer_list);
 
-  for (source_buffer_list_t::const_iterator i = source_buffer_list.begin(); 
+  auto control = _controller.take_control();
+  for (source_buffer_list_t::const_iterator i = source_buffer_list.begin();
        i != source_buffer_list.end(); i++)
   {
-    _controller.set_source_mute(i->id, false);
+    control->source_mute(_scene.string_id(i->id), false);
   }
 }
 
 /// This slot is called when the \a processing \a button was clicked by the user.
 void ssr::QUserInterface::_processing_button_pressed()
 {
-  if(_scene.get_processing_state())
-  {
-    _controller.stop_processing();
-  }
-  else
-  {
-    _controller.start_processing();
-  }
+  _controller.take_control()->processing(!_scene.get_processing_state());
 }
 
 /// This slot is called when the \a pause \a button was clicked by the user.
 void ssr::QUserInterface::_pause_button_pressed()
 {
-  _controller.transport_stop();
+  _controller.take_control()->transport_rolling(false);
 }
 
 /// This slot is called when the \a play \a button was clicked by the user.
 void ssr::QUserInterface::_play_button_pressed()
 {
-  _controller.transport_start();
+  _controller.take_control()->transport_rolling(true);
 }
 
 /** This function is called whenever the fiel menu actions (open/close etc.)
@@ -422,7 +427,7 @@ void ssr::QUserInterface::_show_file_menu()
 
 void ssr::QUserInterface::_new_scene()
 {
-  WARNING("Not implemented yet.");
+  SSR_WARNING("Not implemented yet.");
 }
 
 /** This function reads the file \a _scene_menu.conf and creates the accoding
@@ -449,11 +454,11 @@ void ssr::QUserInterface::_create_scene_menu(const std::string& path_to_scene_me
 
   if (!config_file.is_open())
   {
-    WARNING("Cannot open scene config file '" << path_to_scene_menu << "'.");
+    SSR_WARNING("Cannot open scene config file '" << path_to_scene_menu << "'.");
     return;
   }
 
-  VERBOSE("Creating scene menu from file '" << path_to_scene_menu << "'.");
+  SSR_VERBOSE("Creating scene menu from file '" << path_to_scene_menu << "'.");
 
   std::string line;
   std::string::size_type index;
@@ -488,7 +493,7 @@ void ssr::QUserInterface::_create_scene_menu(const std::string& path_to_scene_me
 
       // Add button to list
       _scene_button_list.push_back(button_buffer);
-      
+
       // increment _scene counter
       no_of_scenes++;
     } // if
@@ -521,7 +526,7 @@ void ssr::QUserInterface::_set_master_volume(float volume)
   volume = std::max(volume, MINVOLUME);
 
   // convert to linear scale
-  _controller.set_master_volume(apf::math::dB2linear(volume));
+  _controller.take_control()->master_volume(apf::math::dB2linear(volume));
 }
 
 /** Changes selected sources' volume.
@@ -529,6 +534,7 @@ void ssr::QUserInterface::_set_master_volume(float volume)
  */
 void ssr::QUserInterface::_change_volume_of_selected_sources(float d_volume)
 {
+  auto control = _controller.take_control();
 
   for (selected_sources_map_t::iterator i = _selected_sources_map.begin(); i != _selected_sources_map.end(); i++)
   {
@@ -538,35 +544,35 @@ void ssr::QUserInterface::_change_volume_of_selected_sources(float d_volume)
     current_gain = std::min(current_gain, dB2linear(MAXVOLUME));
     current_gain = std::max(current_gain, dB2linear(MINVOLUME));
 
-    _controller.set_source_gain(i->second, current_gain);
-   
+    control->source_volume(_scene.string_id(i->second), current_gain);
+
   } // for
 }
 
 /// Opens a save-file-as dialog.
 void ssr::QUserInterface::_save_file_as()
 {
-  QString file_name = QFileDialog::getSaveFileName(this, 
+  QString file_name = QFileDialog::getSaveFileName(this,
                         "Save scene in ASDF", ".", "ASDF files (*.asd)");
 
   // if aborted
-  if ( file_name.isEmpty() ) 
-  { 
-    VERBOSE("Scene not saved.");
-  } 
-  else 
+  if ( file_name.isEmpty() )
+  {
+    SSR_VERBOSE("Scene not saved.");
+  }
+  else
   {
     // convert to std::string
     std::string file_name_std = file_name.toStdString();
 
     // check file extension
-    if ( posixpathtools::get_file_extension(file_name_std) != "asd" )
+    if ( fs::path{file_name_std}.extension() != ".asd" )
     {
       file_name_std.append(".asd");
     }
 
-    _controller.save_scene_as_XML(file_name_std);
-    VERBOSE("Scene saved in '" << file_name_std << "'.");
+    _controller.take_control()->save_scene(file_name_std);
+    SSR_VERBOSE("Scene saved in '" << file_name_std << "'.");
   }
 
 }
@@ -601,7 +607,7 @@ void ssr::QUserInterface::_load_scene(const QString& path_to_scene)
     }
   }
 
-  _controller.load_scene(std::string(path_to_scene.toUtf8()));
+  _controller.take_control()->load_scene(std::string(path_to_scene.toUtf8()));
 
   // clear mouse cursor
   setCursor(Qt::ArrowCursor);
@@ -644,9 +650,9 @@ void ssr::QUserInterface::_update_screen()
     {
       // move the dialog to the desired position
       _update_source_properties_position();
-      
+
       // update displays of source properties dialog
-      _source_properties->update_displays(_scene.get_source(_id_of_last_clicked_source), 
+      _source_properties->update_displays(_scene.get_source(_id_of_last_clicked_source),
 					  _scene.get_reference());
     } // if
 
@@ -667,10 +673,10 @@ void ssr::QUserInterface::_resizeControls(int newWidth)
   const int _time_line_position_x = DEFAULTFRAMELEFT+FILEMENUWIDTH+
   4*(BETWEENBUTTONSPACE+BUTTONWIDTH)+
   BETWEENLABELSPACE - 12; // the 12 is due to LEFTMARGIN in qssrtimeline.cpp
-  
+
   const int _zoom_label_position_x = newWidth - DEFAULTFRAMERIGHT + 3 -
   FILEMENUWIDTH - 2*BUTTONWIDTH - 2*BETWEENLABELSPACE;
-  
+
   // if there is space then show the _time_line
   if (width()-DEFAULTFRAMERIGHT-FILEMENUWIDTH-150 > _time_line_position_x + 30)
   {
@@ -679,13 +685,13 @@ void ssr::QUserInterface::_resizeControls(int newWidth)
     _time_line->show();
   }
   else _time_line->hide();
-  
+
   // if there is space then show the _zoom_label
   if (_zoom_label_position_x > _time_line_position_x - BETWEENLABELSPACE)
   {
     _zoom_label->setGeometry(_zoom_label_position_x, 30, BUTTONWIDTH, 15);
     _zoom_label_text_tag->setGeometry(_zoom_label_position_x, 15, BUTTONWIDTH, 15);
-    
+
     _zoom_label->show();
     _zoom_label_text_tag->show();
   }
@@ -694,7 +700,7 @@ void ssr::QUserInterface::_resizeControls(int newWidth)
     _zoom_label->hide();
     _zoom_label_text_tag->hide();
   }
-  
+
   // if there is space then show the _cpu_label
   if (_zoom_label_position_x > _time_line_position_x - 2*BETWEENLABELSPACE - BUTTONWIDTH)
   {
@@ -702,22 +708,22 @@ void ssr::QUserInterface::_resizeControls(int newWidth)
                             30, BUTTONWIDTH, 15);
     _cpu_label_text_tag->setGeometry(_zoom_label_position_x+BUTTONWIDTH+
                                      BETWEENLABELSPACE, 15, BUTTONWIDTH, 15 );
-    
+
     _cpu_label->show();
     _cpu_label_text_tag->show();
-    
+
   }
   else
   {
     _cpu_label->hide();
     _cpu_label_text_tag->hide();
   }
-  
+
   _volume_slider->setGeometry(newWidth - DEFAULTFRAMERIGHT - FILEMENUWIDTH + 3, 30, FILEMENUWIDTH, 25);
   _volume_slider_text_tag->setGeometry(newWidth - DEFAULTFRAMERIGHT - FILEMENUWIDTH + 3, 15, FILEMENUWIDTH, 15 );
-  
+
   int no_of_scenes = 0;
-  
+
   // check which scene buttons are visible
   if (!_scene_button_list.empty())
   {
@@ -730,19 +736,19 @@ void ssr::QUserInterface::_resizeControls(int newWidth)
         (*i)->show();
       }
       else (*i)->hide();
-      
+
       no_of_scenes++;
     }
   }
-}  
+}
 
 /** Handles Qt mouse press events.
  * @param event Qt mouse event.
  */
 void ssr::QUserInterface::mousePressEvent(QMouseEvent *event)
 {
-  ssr::id_t _id_of_lastlast_clicked_source = _id_of_last_clicked_source;
-  
+  unsigned int _id_of_lastlast_clicked_source = _id_of_last_clicked_source;
+
   event->accept();
 
   _volume_slider_selected = false;
@@ -753,7 +759,7 @@ void ssr::QUserInterface::mousePressEvent(QMouseEvent *event)
 
   uint selected_object = _find_selected_object(event->pos());
 
-  // WARNING("Object " << selected_object << " selected.");
+  // SSR_WARNING("Object " << selected_object << " selected.");
 
   // get position of mouse event
   GLdouble pos_x, pos_y, pos_z;
@@ -783,7 +789,7 @@ void ssr::QUserInterface::mousePressEvent(QMouseEvent *event)
   {
     // hide source properties dialog
     _source_properties->hide();
-    
+
     // no source was clicked
     if (event->button() == Qt::LeftButton) _deselect_all_sources();
     return;
@@ -870,25 +876,27 @@ void ssr::QUserInterface::mouseMoveEvent(QMouseEvent *event)
     Position d_position = position -
       *_scene.get_source_position(_id_of_last_clicked_source);
 
+    auto control = _controller.take_control();
+
     // move all selected sources
     for (selected_sources_map_t::iterator i = _selected_sources_map.begin(); i != _selected_sources_map.end(); i++)
     {
-      // rotate complex sources by the appropriate angle 
-      if (_scene.get_source_model(i->second) == Source::directional ||
-               _scene.get_source_model(i->second) == Source::extended)
+      // rotate complex sources by the appropriate angle
+      if (_scene.get_source_model(i->second) == LegacySource::directional ||
+               _scene.get_source_model(i->second) == LegacySource::extended)
       {
         // position delta expressed as angle
         Orientation d_orientation = (*_scene.get_source_position(i->second) + d_position).orientation() -
             (*_scene.get_source_position(i->second)).orientation();
 
         // set the new orientation
-        _controller.set_source_orientation(i->second,
+        control->source_rotation(_scene.string_id(i->second),
                 (*_scene.get_source_orientation(i->second)) + d_orientation);
       } // if
 
       // finally set the source's position
       // plane waves and point sources will automatically face the reference
-      _controller.set_source_position(i->second,
+      control->source_position(_scene.string_id(i->second),
                 *_scene.get_source_position(i->second) + d_position);
 
     } // for
@@ -918,14 +926,16 @@ void ssr::QUserInterface::mouseMoveEvent(QMouseEvent *event)
     // previous position relative to source position
     prev_mouse_pos -= *_scene.get_source_position(_id_of_last_clicked_source);
 
+    auto control = _controller.take_control();
+
     // rotate all selected sources that can be rotated
     for (selected_sources_map_t::iterator i = _selected_sources_map.begin();
          i != _selected_sources_map.end(); i++)
     {
-      if (_scene.get_source_model(i->second) == Source::directional ||
-          _scene.get_source_model(i->second) == Source::extended)
+      if (_scene.get_source_model(i->second) == LegacySource::directional ||
+          _scene.get_source_model(i->second) == LegacySource::extended)
       {
-        _controller.set_source_orientation(i->second,
+        control->source_rotation(_scene.string_id(i->second),
                 (*_scene.get_source_orientation(i->second)) +
                 (mouse_pos.orientation() - prev_mouse_pos.orientation()));
       }
@@ -956,7 +966,7 @@ void ssr::QUserInterface::mouseMoveEvent(QMouseEvent *event)
       // change gain
       if (gain > MINVOLUME && gain <= MAXVOLUME)
       {
-        _controller.set_source_gain(i->second, dB2linear(gain));
+        _controller.take_control()->source_volume(_scene.string_id(i->second), dB2linear(gain));
       }
     } // for
 
@@ -985,7 +995,7 @@ void ssr::QUserInterface::mouseMoveEvent(QMouseEvent *event)
     // previous position relative to relative position
     prev_mouse_pos -= _scene.get_reference().position;
 
-    _controller.set_reference_orientation(_scene.get_reference().orientation +
+    _controller.take_control()->reference_rotation(_scene.get_reference().orientation +
                                           (mouse_pos.orientation() - prev_mouse_pos.orientation()));
 
   } // else if
@@ -1001,7 +1011,7 @@ void ssr::QUserInterface::mouseMoveEvent(QMouseEvent *event)
     position.x = static_cast<float>(pos_x + _x_offset);
     position.y = static_cast<float>(pos_y + _y_offset);
 
-    _controller.set_reference_position(position);
+    _controller.take_control()->reference_position(position);
   } // else if
 
   // right click on background
@@ -1066,8 +1076,8 @@ void ssr::QUserInterface::mouseDoubleClickEvent(QMouseEvent *event)
     // restore zoom
     _set_zoom(100);
   }
-  else 
-  { 
+  else
+  {
     // double click on source
     if (_source_properties->isVisible())
     {
@@ -1128,32 +1138,35 @@ void ssr::QUserInterface::keyPressEvent(QKeyEvent *event)
   case Qt::Key_Down: _window_y_offset += 0.1f; update(); break;
   case Qt::Key_Left: _window_x_offset += 0.1f; update(); break;
   case Qt::Key_Right: _window_x_offset -= 0.1f; update(); break;
-  case Qt::Key_Space: if (_scene.is_playing()) _controller.transport_stop();
-                      else _controller.transport_start();
-                      break;
+  case Qt::Key_Space: _controller.take_control()->transport_rolling(!_scene.is_playing()); break;
   case Qt::Key_Backspace: _skip_back(); break;
 
     // actions related to sources
   case Qt::Key_A: if (event->modifiers() == Qt::ControlModifier) _select_all_sources(); break;
   case Qt::Key_F: _toggle_fixation_state_of_selected_sources(); break;
   case Qt::Key_M: _toggle_mute_state_of_selected_sources(); break;
-  case Qt::Key_P: _toggle_source_models(); break;  
-  case Qt::Key_R: _controller.set_auto_rotation(!_scene.get_auto_rotation()); break;
+  case Qt::Key_P: _toggle_source_models(); break;
+  case Qt::Key_R: _controller.take_control()->auto_rotate_sources(!_scene.get_auto_rotation()); break;
 
-  case Qt::Key_S: if ( event->modifiers() == Qt::ControlModifier ) 
+  case Qt::Key_S: if ( event->modifiers() == Qt::ControlModifier )
                   {
                     _save_file_as();
                   }
-                  else if (_selected_sources_map.empty()) 
+                  else if ( event->modifiers() == Qt::ShiftModifier )
+                  {
+                    _unsolo_all_sources();
+                    _toggle_solo_state_of_selected_sources();
+                  }
+                  else if (_selected_sources_map.empty())
                   {
                     _unsolo_all_sources();
                   }
 		  else _toggle_solo_state_of_selected_sources();
                   break;
 
-  case Qt::Key_T: if ( event->modifiers() == Qt::ControlModifier ) 
+  case Qt::Key_T: if ( event->modifiers() == Qt::ControlModifier )
                   {
-		    _time_line->show_time_edit(); 
+		    _time_line->show_time_edit();
                   }
                   break;
 
@@ -1170,23 +1183,24 @@ void ssr::QUserInterface::keyPressEvent(QKeyEvent *event)
 
     // miscellaneous actions
   case Qt::Key_Plus: if (_selected_sources_map.empty())
-		     { 
+		     {
 		       // change master level
 		       _set_master_volume(linear2dB(_scene.get_master_volume() + 0.00001f) + 1.0f); // min -100dB
 		     }
                      // change selected sources level
 		     else _change_volume_of_selected_sources(1.0f);
-                     break;   
+                     break;
   case Qt::Key_Minus: if (_selected_sources_map.empty())
-		     { 
+		     {
 		       // change master level
 		       _set_master_volume(linear2dB(_scene.get_master_volume() + 0.00001f) - 1.0f); // min -100dB
 		     }
                      // change selected sources level
 		     else _change_volume_of_selected_sources(-1.0f);
-                     break; 
-  case Qt::Key_Return: {_controller.calibrate_client(); break; }
+                     break;
+  case Qt::Key_Return: {_controller.take_control()->reset_tracker(); break; }
   case Qt::Key_Control: {_ctrl_pressed = true; break; }
+  case Qt::Key_Shift: {_shift_pressed = true; break; }
   case Qt::Key_Alt: {_alt_pressed = true; break; }
   case Qt::Key_F11: {if (!isFullScreen()) setWindowState(Qt::WindowFullScreen);
       else setWindowState( Qt::WindowNoState );} break;
@@ -1194,7 +1208,7 @@ void ssr::QUserInterface::keyPressEvent(QKeyEvent *event)
         close();
       else
         break;}
-  case Qt::Key_Escape: close();
+  case Qt::Key_Q: {if (_ctrl_pressed) close();}
   } // switch
 }
 
@@ -1206,6 +1220,7 @@ void ssr::QUserInterface::keyReleaseEvent(QKeyEvent *event)
     switch (event->key()){
     case Qt::Key_Control: {_ctrl_pressed = false; break; }
     case Qt::Key_Alt: {_alt_pressed = false; break; }
+    case Qt::Key_Shift: {_shift_pressed = false; break; }
     default: ;
     }
 }
@@ -1222,6 +1237,52 @@ void ssr::QUserInterface::wheelEvent(QWheelEvent *event)
                              (100.0f+event->delta()/100.f*5.0f) + 0.5f));
 }
 
+/** Handles Qt drag & drop events.
+ * @param event Qt mouse drag enter event.
+ */
+void ssr::QUserInterface::dragEnterEvent(QDragEnterEvent *event)
+{
+  // make sure that the drag event contains file paths
+  if (event->mimeData()->hasUrls()) event->acceptProposedAction();
+  //if (event->mimeData()->hasFormat("text/plain"))
+   // event->acceptProposedAction();
+}
+
+/** Handles Qt drag & drop events.
+ * @param event Qt mouse drop event.
+ */
+void ssr::QUserInterface::dropEvent(QDropEvent *event)
+{
+  event->acceptProposedAction();
+
+  const QMimeData* mimeData = event->mimeData();
+
+  // check for our needed mime type, here a file or a list of files
+  if (mimeData->hasUrls())
+  {
+    QList<QUrl> urlList = mimeData->urls();
+
+    QUrl url(urlList.at(0));
+
+    SSR_VERBOSE("Dropped file: " <<
+      std::string(url.toString().toStdString()));
+
+    // Remove "file://"
+    //this->_load_scene(url.toString().toStdString().substr(7));
+    this->_load_scene(url.toString().remove(0,7));
+
+    // TODO: Add source if more than one file is dragged and dropped.
+    //QStringList pathList;
+    // for (int i = 0; i < urlList.size() && i < 32; +i)
+    //{
+    //  pathList.append(urlList.at(i).toLocalFile());
+    //}
+    // call a function to open all files (needs function is yet to be written)
+    //openFiles(pathList);
+
+   }
+}
+
 /// Displays the about window.
 void ssr::QUserInterface::_show_about_window()
 {
@@ -1269,7 +1330,7 @@ void ssr::QUserInterface::_show_about_window()
 
   // text_label.setGeometry(0, 261, 350, 370);
   text_label.setText(about_string.c_str());
-  text_label.setIndent(20); 
+  text_label.setIndent(20);
   text_label.setAlignment(Qt::AlignTop);
   text_label.adjustSize();
 
@@ -1285,7 +1346,7 @@ void ssr::QUserInterface::_show_about_window()
 
   connect(&ssr_logo, SIGNAL(clicked()), &about_window, SLOT(close()));
   connect(&text_label, SIGNAL(clicked()), &about_window, SLOT(close()));
- 
+
   about_window.exec();
 }
 
@@ -1294,17 +1355,18 @@ void ssr::QUserInterface::_show_about_window()
  */
 void ssr::QUserInterface::_set_source_mute(const bool flag)
 {
-  _controller.set_source_mute(_id_of_last_clicked_source, flag);
+  _controller.take_control()->source_mute(_scene.string_id(_id_of_last_clicked_source), flag);
 }
 
 /// Toggles the mute state of all selected sound sources
 void ssr::QUserInterface::_toggle_mute_state_of_selected_sources()
 {
+  auto control = _controller.take_control();
   // iterate over selected sources
   for (  selected_sources_map_t::iterator i = _selected_sources_map.begin();
          i != _selected_sources_map.end(); i++)
   {
-    _controller.set_source_mute(i->second,
+    control->source_mute(_scene.string_id(i->second),
                                 !_scene.get_source_mute_state(i->second));
   }
 }
@@ -1315,6 +1377,8 @@ void ssr::QUserInterface::_toggle_solo_state_of_selected_sources()
   source_buffer_list_t source_buffer_list;
   _scene.get_sources(source_buffer_list);
 
+  auto control = _controller.take_control();
+
   for (selected_sources_map_t::iterator i = _selected_sources_map.begin();
        i != _selected_sources_map.end(); i++)
   {
@@ -1323,7 +1387,7 @@ void ssr::QUserInterface::_toggle_solo_state_of_selected_sources()
       // solo source
       _soloed_sources.insert(i->second);
       // make sure it's not muted
-      _controller.set_source_mute(i->second, false);
+      control->source_mute(_scene.string_id(i->second), false);
 
       // mute other sources
       for (source_buffer_list_t::const_iterator j = source_buffer_list.begin();
@@ -1333,7 +1397,7 @@ void ssr::QUserInterface::_toggle_solo_state_of_selected_sources()
         if (_soloed_sources.find(j->id) == _soloed_sources.end())
         {
           // then mute it
-          _controller.set_source_mute(j->id, true);
+          control->source_mute(_scene.string_id(j->id), true);
         }
       }
     }
@@ -1345,7 +1409,7 @@ void ssr::QUserInterface::_toggle_solo_state_of_selected_sources()
       for (source_buffer_list_t::const_iterator j = source_buffer_list.begin();
       j != source_buffer_list.end(); j++)
       {
-        _controller.set_source_mute(j->id, false);
+        control->source_mute(_scene.string_id(j->id), false);
       }
     }
   } // for
@@ -1353,63 +1417,63 @@ void ssr::QUserInterface::_toggle_solo_state_of_selected_sources()
 
 void ssr::QUserInterface::_toggle_source_models()
 {
+  auto control = _controller.take_control();
   // toggle all source types between "plane" and "point"
   for (selected_sources_map_t::iterator i = _selected_sources_map.begin(); i != _selected_sources_map.end(); i++)
   {
     // if source is plane then make sure that it faces the reference
-    if (_scene.get_source_model(i->second) == Source::plane)
+    if (_scene.get_source_model(i->second) == LegacySource::plane)
     {
-      _controller.set_source_model(i->second, Source::point);
+      control->source_model(_scene.string_id(i->second), apf::str::A2S(LegacySource::point));
     }
-    else if (_scene.get_source_model(i->second) == Source::point)
+    else if (_scene.get_source_model(i->second) == LegacySource::point)
     {
-      _controller.set_source_model(i->second, Source::plane);      
+      control->source_model(_scene.string_id(i->second), apf::str::A2S(LegacySource::plane));
     } // if
   } // for
 }
 
-/** Sets the position fixed state of the currently active mouse.  
+/** Sets the position fixed state of the currently active mouse.
  * @param flag \a true or \a false
  */
 void ssr::QUserInterface::_set_source_position_fixed(const bool flag)
 {
-  _controller.set_source_position_fixed(_id_of_last_clicked_source, flag);
+  _controller.take_control()->source_fixed(_scene.string_id(_id_of_last_clicked_source), flag);
 }
 
-/** Sets the position fixed state of the currently active mouse.  
- * @param index \a 0="plane wave" \a 1="point source" 
+/** Sets the position fixed state of the currently active mouse.
+ * @param index \a 0="plane wave" \a 1="point source"
  */
 void ssr::QUserInterface::_set_source_model(const int index)
 {
-  Source::model_t model = Source::unknown;
-  
+  LegacySource::model_t model = LegacySource::unknown;
+
   switch(index){
     case 0:
-      model = Source::plane;
+      model = LegacySource::plane;
       break;
     case 1:
-      model = Source::point;
+      model = LegacySource::point;
       break;
   }
-  VERBOSE("index: " << index);
+  SSR_VERBOSE("index: " << index);
 
-  ssr::id_t id = _id_of_last_clicked_source;
-  _controller.set_source_model(id, model);
+  unsigned int id = _id_of_last_clicked_source;
+  auto control = _controller.take_control();
+  control->source_model(_scene.string_id(id), apf::str::A2S(model));
   // make sure that the plane wave is oriented towards the reference point
-  _controller.set_source_orientation(id, (_scene.get_reference().position
+  control->source_rotation(_scene.string_id(id), (_scene.get_reference().position
         - *_scene.get_source_position(id)).orientation());
 }
 
 void ssr::QUserInterface::_toggle_fixation_state_of_selected_sources()
 {
+  auto control = _controller.take_control();
   // iterate over selected sources
   for (  selected_sources_map_t::iterator i = _selected_sources_map.begin();
          i != _selected_sources_map.end(); i++)
   {
-    _controller.set_source_position_fixed(i->second,
+    control->source_fixed(_scene.string_id(i->second),
                                 !_scene.get_source_position_fixed(i->second));
   }
 }
-
-// Settings for Vim (http://www.vim.org/), please do not remove:
-// vim:softtabstop=2:shiftwidth=2:expandtab:textwidth=80:cindent
