@@ -39,10 +39,8 @@
 #include "apf/parameter_map.h"
 #include "apf/math.h"  // for dB2linear()
 
-// TODO: avoid multiple ambiguous "Source" classes
-#include "source.h"  // for ::Source::model_t
-
 #include "maptools.h"
+#include "geometry.h"  // for vec3
 
 #ifndef SSR_QUERY_POLICY
 #define SSR_QUERY_POLICY apf::disable_queries
@@ -52,8 +50,6 @@ namespace ssr
 {
 
 /** Renderer base class.
- * @todo more documentation!
- *
  * The parallel rendering engine uses the non-blocking datastructure RtList to
  * communicate between realtime and non-realtime threads.
  * All non-realtime accesses to RtList%s have to be locked with
@@ -62,12 +58,11 @@ namespace ssr
 template<typename Derived>
 class RendererBase : public apf::MimoProcessor<Derived
                      , APF_MIMOPROCESSOR_INTERFACE_POLICY
-                     , APF_MIMOPROCESSOR_THREAD_POLICY
                      , SSR_QUERY_POLICY>
 {
   private:
     using _base = apf::MimoProcessor<Derived, APF_MIMOPROCESSOR_INTERFACE_POLICY
-      , APF_MIMOPROCESSOR_THREAD_POLICY, SSR_QUERY_POLICY>;
+      , SSR_QUERY_POLICY>;
 
   public:
     using typename _base::rtlist_t;
@@ -116,9 +111,9 @@ class RendererBase : public apf::MimoProcessor<Derived
     {
       State(apf::CommandQueue& fifo, const apf::parameter_map& params)
         : reference_position(fifo)
-        , reference_orientation(fifo, Orientation(90))
-        , reference_offset_position(fifo)
-        , reference_offset_orientation(fifo)
+        , reference_rotation(fifo)
+        , reference_position_offset(fifo)
+        , reference_rotation_offset(fifo)
         , master_volume(fifo, 1)
         , processing(fifo, true)
         , decay_exponent(fifo, params.get<sample_type>("decay_exponent", 1))
@@ -126,10 +121,10 @@ class RendererBase : public apf::MimoProcessor<Derived
             , params.get<sample_type>("amplitude_reference_distance", 3))
       {}
 
-      apf::SharedData<Position> reference_position;
-      apf::SharedData<Orientation> reference_orientation;
-      apf::SharedData<Position> reference_offset_position;
-      apf::SharedData<Orientation> reference_offset_orientation;
+      apf::SharedData<Pos> reference_position;
+      apf::SharedData<Rot> reference_rotation;
+      apf::SharedData<Pos> reference_position_offset;
+      apf::SharedData<Rot> reference_rotation_offset;
       apf::SharedData<sample_type> master_volume;
       apf::SharedData<bool> processing;
       apf::SharedData<sample_type> decay_exponent;
@@ -203,11 +198,12 @@ class RendererBase : public apf::MimoProcessor<Derived
             input, output, member));
     }
 
-    int add_source(const apf::parameter_map& p = apf::parameter_map());
-    void rem_source(int id);
+    std::string add_source(id_t id
+        , const apf::parameter_map& p = apf::parameter_map());
+    void rem_source(id_t id);
     void rem_all_sources();
 
-    Source* get_source(int id);
+    Source* get_source(id_t id);
 
     // May only be used in realtime thread!
     const rtlist_t& get_source_list() const { return _source_list; }
@@ -216,13 +212,9 @@ class RendererBase : public apf::MimoProcessor<Derived
 
     // TODO: proper solution for getting the reproduction setup
     template<typename SomeListType>
-    void get_loudspeakers(SomeListType&) {}
+    void get_loudspeakers(SomeListType&) const {}
 
-    std::unique_ptr<ScopedLock> get_scoped_lock()
-    {
-      // TODO: in C++14, use make_unique()
-      return std::unique_ptr<ScopedLock>(new ScopedLock(_lock));
-    }
+    auto get_scoped_lock() { return std::make_unique<ScopedLock>(_lock); }
 
     const sample_type master_volume_correction;  // linear
 
@@ -245,13 +237,11 @@ class RendererBase : public apf::MimoProcessor<Derived
       return temp;
     }
 
-    int _get_new_id();
+    std::map<std::string, Source*, std::less<>> _source_map;
 
-    std::map<int, Source*> _source_map;
+    size_t _next_id_suffix = 0;
 
-    int _highest_id;
-
-    typename _base::Lock _lock;
+    std::mutex _lock;
 };
 
 /** Constructor.
@@ -266,7 +256,6 @@ RendererBase<Derived>::RendererBase(const apf::parameter_map& p)
   , _master_level()
   , _source_list(_fifo)
   , _show_head(true)
-  , _highest_id(0)
 {}
 
 /** Create a new source.
@@ -274,13 +263,26 @@ RendererBase<Derived>::RendererBase(const apf::parameter_map& p)
  * @throw unknown whatever the Derived::Source constructor throws
  **/
 template<typename Derived>
-int RendererBase<Derived>::add_source(const apf::parameter_map& p)
+std::string
+RendererBase<Derived>::add_source(id_t requested_id
+    , const apf::parameter_map& p)
 {
-  int id = _get_new_id();
+  std::string id = requested_id;
+  if (id.size() && _source_map.find(id) != _source_map.end())
+  {
+    throw std::runtime_error("Source with ID \"" + id + "\" already exists");
+  }
+  if (id == "")
+  {
+    do
+    {
+      id = ".ssr:" + apf::str::A2S(_next_id_suffix++);
+    }
+    while (_source_map.find(id) != _source_map.end());
+  }
 
   typename Derived::Input::Params in_params;
   in_params = p;
-  in_params.set("id", in_params.get("id", id));
   auto in = this->add(in_params);
 
   // WARNING: if Derived::Input throws an exception, the SSR crashes!
@@ -290,8 +292,6 @@ int RendererBase<Derived>::add_source(const apf::parameter_map& p)
   src_params.parent = &this->derived();
   src_params.fifo = &_fifo;
   src_params.input = in;
-
-  // For now, Input ID and Source ID are the same:
   src_params.id = id;
 
   typename Derived::Source* src;
@@ -314,11 +314,10 @@ int RendererBase<Derived>::add_source(const apf::parameter_map& p)
 
   _source_map[id] = src;
   return id;
-  // TODO: what happens on failure? can there be failure?
 }
 
 template<typename Derived>
-void RendererBase<Derived>::rem_source(int id)
+void RendererBase<Derived>::rem_source(id_t id)
 {
   auto delinquent = _source_map.find(id);
 
@@ -351,21 +350,22 @@ void RendererBase<Derived>::rem_all_sources()
   {
     this->rem_source(_source_map.begin()->first);
   }
-  _highest_id = 0;
 }
 
+
 template<typename Derived>
 typename RendererBase<Derived>::Source*
-RendererBase<Derived>::get_source(int id)
+RendererBase<Derived>::get_source(id_t id)
 {
-  return maptools::get_item(_source_map, id);
-}
-
-template<typename Derived>
-int
-RendererBase<Derived>::_get_new_id()
-{
-  return ++_highest_id;
+  auto iter = _source_map.find(id);
+  if (iter == _source_map.end())
+  {
+    return nullptr;
+  }
+  else
+  {
+    return iter->second;
+  }
 }
 
 /// A sound source.
@@ -389,7 +389,7 @@ class RendererBase<Derived>::Source
       Derived* parent = nullptr;
       const typename Derived::Input* input = nullptr;
       apf::CommandQueue* fifo = nullptr;
-      int id = 0;
+      std::string id;
 
       using apf::parameter_map::operator=;
     };
@@ -397,12 +397,13 @@ class RendererBase<Derived>::Source
     explicit Source(const Params& p)
       : parent(*(p.parent ? p.parent : throw std::logic_error(
               "Bug (RendererBase::Source): parent == NULL!")))
+      , active(*p.fifo, false)
       , position(*(p.fifo ? p.fifo : throw std::logic_error(
               "Bug (RendererBase::Source): fifo == NULL!")))
-      , orientation(*p.fifo)
+      , rotation(*p.fifo)
       , gain(*p.fifo, sample_type(1.0))
       , mute(*p.fifo, false)
-      , model(*p.fifo, ::Source::point)
+      , model(*p.fifo, "point")
       , weighting_factor()
       , id(p.id)
       , _input(*(p.input ? p.input : throw std::logic_error(
@@ -426,15 +427,16 @@ class RendererBase<Derived>::Source
 
     Derived& parent;
 
-    apf::SharedData<Position> position;
-    apf::SharedData<Orientation> orientation;
+    apf::SharedData<bool> active;
+    apf::SharedData<Pos> position;
+    apf::SharedData<Rot> rotation;
     apf::SharedData<sample_type> gain;
     apf::SharedData<bool> mute;
-    apf::SharedData< ::Source::model_t> model;
+    apf::SharedData<std::string> model;
 
     apf::BlockParameter<sample_type> weighting_factor;
 
-    const int id;
+    const std::string id;
 
   protected:
     const typename Derived::Input& _input;
@@ -444,7 +446,8 @@ class RendererBase<Derived>::Source
 
     void _level_helper(apf::enable_queries&)
     {
-      _pre_fader_level = apf::math::max_amplitude(_input.begin(), _input.end());
+      _pre_fader_level
+        = apf::math::max_amplitude(this->begin(), this->end());
       _level = _pre_fader_level * this->weighting_factor;
     }
 
@@ -460,7 +463,7 @@ void RendererBase<Derived>::Source::_process()
   this->_begin = _input.begin();
   this->_end = _input.end();
 
-  if (!_input.parent.state.processing || this->mute)
+  if (!this->parent.state.processing || this->mute || !this->active)
   {
     this->weighting_factor = 0.0;
   }
@@ -469,37 +472,37 @@ void RendererBase<Derived>::Source::_process()
     this->weighting_factor = this->gain;
     // If the renderer does something nonlinear, the master volume should
     // be applied to the output signal ... TODO: shall we care?
-    this->weighting_factor *= _input.parent.state.master_volume;
-    this->weighting_factor *= _input.parent.master_volume_correction;
+    this->weighting_factor *= this->parent.state.master_volume;
+    this->weighting_factor *= this->parent.master_volume_correction;
 
     // apply distance attenuation
     if (std::strcmp(this->parent.name(), "BrsRenderer") != 0
          && std::strcmp(this->parent.name(), "GenericRenderer") != 0)
-    {    
-      if (this->model != ::Source::plane)
+    {
+      if (this->model != "plane")
       {
-        float source_distance = (this->position
-          - (_input.parent.state.reference_position
-            + _input.parent.state.reference_offset_position)).length();
-      
+        float source_distance = length(vec3{this->position}
+          - (vec3{this->parent.state.reference_position}
+            + vec3{this->parent.state.reference_position_offset}));
+
         // no volume increase for sources closer than 0.5 m
         source_distance = std::max(source_distance, 0.5f);
 
        // standard 1/r: weight *= 1.0f / source_distance;
-       this->weighting_factor *= 1.0f 
-         / pow(source_distance, _input.parent.state.decay_exponent); // 1/r^e
+       this->weighting_factor *= 1.0f
+         / pow(source_distance, this->parent.state.decay_exponent); // 1/r^e
 
        // plane wave always have the same amplitude independent of the amplitude
        // reference distance and the decay exponent; normalize all other sources
-       // accordingly 
-       this->weighting_factor *= 
-         pow(_input.parent.state.amplitude_reference_distance,
-           _input.parent.state.decay_exponent);
+       // accordingly
+       this->weighting_factor *=
+         pow(this->parent.state.amplitude_reference_distance,
+           this->parent.state.decay_exponent);
       } // if model::plane
     } // if != BRS or Generic
   } // if muted or not
 
-  _level_helper(_input.parent);
+  _level_helper(this->parent);
 
   assert(this->weighting_factor.exactly_one_assignment());
 }
@@ -600,6 +603,3 @@ struct SourceToOutput : Base<Derived>
 }  // namespace ssr
 
 #endif
-
-// Settings for Vim (http://www.vim.org/), please do not remove:
-// vim:softtabstop=2:shiftwidth=2:expandtab:textwidth=80:cindent
